@@ -13,6 +13,7 @@ use crate::tab::tab_style;
 pub struct LinePart {
     part: String,
     len: usize,
+    tab_index: Option<usize>,
 }
 
 #[derive(Default)]
@@ -21,7 +22,7 @@ struct State {
     active_tab_idx: usize,
     mode_info: ModeInfo,
     mouse_click_pos: usize,
-    should_render: bool,
+    should_change_tab: bool,
 }
 
 static ARROW_SEPARATOR: &str = "";
@@ -38,13 +39,23 @@ impl ZellijPlugin for State {
         ]);
     }
 
-    fn update(&mut self, event: Event) {
+    fn update(&mut self, event: Event) -> bool {
+        let mut should_render = false;
         match event {
-            Event::ModeUpdate(mode_info) => self.mode_info = mode_info,
+            Event::ModeUpdate(mode_info) => {
+                if self.mode_info != mode_info {
+                    should_render = true;
+                }
+                self.mode_info = mode_info
+            },
             Event::TabUpdate(tabs) => {
                 if let Some(active_tab_index) = tabs.iter().position(|t| t.active) {
                     // tabs are indexed starting from 1 so we need to add 1
-                    self.active_tab_idx = active_tab_index + 1;
+                    let active_tab_idx = active_tab_index + 1;
+                    if self.active_tab_idx != active_tab_idx || self.tabs != tabs {
+                        should_render = true;
+                    }
+                    self.active_tab_idx = active_tab_idx;
                     self.tabs = tabs;
                 } else {
                     eprintln!("Could not find active tab.");
@@ -52,13 +63,18 @@ impl ZellijPlugin for State {
             },
             Event::Mouse(me) => match me {
                 Mouse::LeftClick(_, col) => {
+                    if self.mouse_click_pos != col {
+                        should_render = true;
+                        self.should_change_tab = true;
+                    }
                     self.mouse_click_pos = col;
-                    self.should_render = true;
                 },
                 Mouse::ScrollUp(_) => {
+                    should_render = true;
                     switch_tab_to(min(self.active_tab_idx + 1, self.tabs.len()) as u32);
                 },
                 Mouse::ScrollDown(_) => {
+                    should_render = true;
                     switch_tab_to(max(self.active_tab_idx.saturating_sub(1), 1) as u32);
                 },
                 _ => {},
@@ -66,7 +82,8 @@ impl ZellijPlugin for State {
             _ => {
                 eprintln!("Got unrecognized event: {:?}", event);
             },
-        }
+        };
+        should_render
     }
 
     fn render(&mut self, _rows: usize, cols: usize) {
@@ -88,12 +105,10 @@ impl ZellijPlugin for State {
             }
             let tab = tab_style(
                 tabname,
-                t.active,
+                t,
                 is_alternate_tab,
-                t.is_sync_panes_active,
                 self.mode_info.style.colors,
                 self.mode_info.capabilities,
-                t.other_focused_clients.as_slice(),
             );
             is_alternate_tab = !is_alternate_tab;
             all_tabs.push(tab);
@@ -109,17 +124,17 @@ impl ZellijPlugin for State {
         );
         let mut s = String::new();
         let mut len_cnt = 0;
-        for (idx, bar_part) in tab_line.iter().enumerate() {
+        for bar_part in tab_line {
             s = format!("{}{}", s, &bar_part.part);
 
-            if self.should_render
-                && self.mouse_click_pos > len_cnt
-                && self.mouse_click_pos <= len_cnt + bar_part.len
-                && idx > 3
+            if self.should_change_tab
+                && self.mouse_click_pos >= len_cnt
+                && self.mouse_click_pos < len_cnt + bar_part.len
+                && bar_part.tab_index.is_some()
             {
-                // First three elements of tab_line are "Zellij", session name and mode, hence the idx > 3 condition.
-                // Tabs are indexed starting from 1, therefore we need subtract 3 below.
-                switch_tab_to(TryInto::<u32>::try_into(idx).unwrap() - 3);
+                // Tabs are indexed starting from 1, therefore we need add 1 to tab_index.
+                let tab_index: u32 = bar_part.tab_index.unwrap().try_into().unwrap();
+                switch_tab_to(tab_index + 1);
             }
             len_cnt += bar_part.len;
         }
@@ -129,12 +144,12 @@ impl ZellijPlugin for State {
         };
         match background {
             PaletteColor::Rgb((r, g, b)) => {
-                println!("{}\u{1b}[48;2;{};{};{}m\u{1b}[0K", s, r, g, b);
+                print!("{}\u{1b}[48;2;{};{};{}m\u{1b}[0K", s, r, g, b);
             },
             PaletteColor::EightBit(color) => {
-                println!("{}\u{1b}[48;5;{}m\u{1b}[0K", s, color);
+                print!("{}\u{1b}[48;5;{}m\u{1b}[0K", s, color);
             },
         }
-        self.should_render = false;
+        self.should_change_tab = false;
     }
 }
