@@ -114,6 +114,24 @@ impl InputHandler {
                                     None,
                                 );
                             }
+                            if self.mode == InputMode::EnterSearch {
+                                self.dispatch_action(
+                                    Action::SearchInput(pasted_text.as_bytes().to_vec()),
+                                    None,
+                                );
+                            }
+                            if self.mode == InputMode::RenameTab {
+                                self.dispatch_action(
+                                    Action::TabNameInput(pasted_text.as_bytes().to_vec()),
+                                    None,
+                                );
+                            }
+                            if self.mode == InputMode::RenamePane {
+                                self.dispatch_action(
+                                    Action::PaneNameInput(pasted_text.as_bytes().to_vec()),
+                                    None,
+                                );
+                            }
                         },
                         _ => {},
                     }
@@ -129,6 +147,20 @@ impl InputHandler {
                         self.handle_stdin_ansi_instruction(ansi_instruction);
                     }
                 },
+                Ok((InputInstruction::StartedParsing, _error_context)) => {
+                    log::info!("sending done loading");
+                    self.send_client_instructions
+                        .send(ClientInstruction::StartedParsingStdinQuery)
+                        .unwrap();
+                    log::info!("done sent done loading");
+                },
+                Ok((InputInstruction::DoneParsing, _error_context)) => {
+                    log::info!("sending done loading");
+                    self.send_client_instructions
+                        .send(ClientInstruction::DoneParsingStdinQuery)
+                        .unwrap();
+                    log::info!("done sent done loading");
+                },
                 Err(err) => panic!("Encountered read error: {:?}", err),
             }
         }
