@@ -2,8 +2,8 @@ use super::input_loop;
 use crate::stdin_ansi_parser::StdinAnsiParser;
 use crate::stdin_loop;
 use zellij_utils::anyhow::Result;
-use zellij_utils::data::{InputMode, Palette};
-use zellij_utils::input::actions::{Action, Direction};
+use zellij_utils::data::{Direction, InputMode, Palette};
+use zellij_utils::input::actions::Action;
 use zellij_utils::input::config::Config;
 use zellij_utils::input::options::Options;
 use zellij_utils::nix;
