@@ -1,17 +1,22 @@
 mod pane_resizer;
 mod tiled_pane_grid;
 
+use crate::resize_pty;
+use tiled_pane_grid::{split, TiledPaneGrid, RESIZE_PERCENT};
+
 use crate::{
     os_input_output::ServerOsApi,
     output::Output,
     panes::{ActivePanes, PaneId},
+    plugins::PluginInstruction,
     tab::{Pane, MIN_TERMINAL_HEIGHT, MIN_TERMINAL_WIDTH},
-    ui::{boundaries::Boundaries, pane_contents_and_ui::PaneContentsAndUi},
+    thread_bus::ThreadSenders,
+    ui::boundaries::Boundaries,
+    ui::pane_contents_and_ui::PaneContentsAndUi,
     ClientId,
 };
-use tiled_pane_grid::{split, TiledPaneGrid};
 use zellij_utils::{
-    data::{ModeInfo, Style},
+    data::{ModeInfo, ResizeStrategy, Style},
     errors::prelude::*,
     input::{command::RunCommand, layout::SplitDirection},
     pane_size::{Offset, PaneGeom, Size, SizeInPixels, Viewport},
@@ -24,22 +29,6 @@ use std::{
     time::Instant,
 };
 
-macro_rules! resize_pty {
-    ($pane:expr, $os_input:expr) => {
-        if let PaneId::Terminal(ref pid) = $pane.pid() {
-            // FIXME: This `set_terminal_size_using_terminal_id` call would be best in
-            // `TerminalPane::reflow_lines`
-            $os_input.set_terminal_size_using_terminal_id(
-                *pid,
-                $pane.get_content_columns() as u16,
-                $pane.get_content_rows() as u16,
-            )
-        } else {
-            Ok(())
-        }
-    };
-}
-
 fn pane_content_offset(position_and_size: &PaneGeom, viewport: &Viewport) -> (usize, usize) {
     // (columns_offset, rows_offset)
     // if the pane is not on the bottom or right edge on the screen, we need to reserve one space
@@ -75,6 +64,9 @@ pub struct TiledPanes {
     panes_to_hide: HashSet<PaneId>,
     fullscreen_is_active: bool,
     os_api: Box<dyn ServerOsApi>,
+    senders: ThreadSenders,
+    window_title: Option<String>,
+    client_id_to_boundaries: HashMap<ClientId, Boundaries>,
 }
 
 impl TiledPanes {
@@ -91,6 +83,7 @@ impl TiledPanes {
         default_mode_info: ModeInfo,
         style: Style,
         os_api: Box<dyn ServerOsApi>,
+        senders: ThreadSenders,
     ) -> Self {
         TiledPanes {
             panes: BTreeMap::new(),
@@ -108,6 +101,9 @@ impl TiledPanes {
             panes_to_hide: HashSet::new(),
             fullscreen_is_active: false,
             os_api,
+            senders,
+            window_title: None,
+            client_id_to_boundaries: HashMap::new(),
         }
     }
     pub fn add_pane_with_existing_geom(&mut self, pane_id: PaneId, mut pane: Box<dyn Pane>) {
@@ -270,8 +266,9 @@ impl TiledPanes {
                 pane.set_content_offset(Offset::shift(pane_rows_offset, pane_columns_offset));
             }
 
-            resize_pty!(pane, self.os_api).unwrap();
+            resize_pty!(pane, self.os_api, self.senders).unwrap();
         }
+        self.reset_boundaries();
     }
     pub fn can_split_pane_horizontally(&mut self, client_id: ClientId) -> bool {
         if let Some(active_pane_id) = &self.active_panes.get(&client_id) {
@@ -346,9 +343,16 @@ impl TiledPanes {
                     .insert(client_id, pane_id, &mut self.panes);
             }
         }
+        self.reset_boundaries();
+    }
+    pub fn focus_pane_if_client_not_focused(&mut self, pane_id: PaneId, client_id: ClientId) {
+        if self.active_panes.get(&client_id).is_none() {
+            self.focus_pane(pane_id, client_id)
+        }
     }
     pub fn clear_active_panes(&mut self) {
         self.active_panes.clear(&mut self.panes);
+        self.reset_boundaries();
     }
     pub fn first_active_pane_id(&self) -> Option<PaneId> {
         self.connected_clients
@@ -381,6 +385,7 @@ impl TiledPanes {
             pane.set_should_render_boundaries(true);
             pane.render_full_viewport();
         }
+        self.reset_boundaries();
     }
     pub fn has_active_panes(&self) -> bool {
         !self.active_panes.is_empty()
@@ -443,7 +448,11 @@ impl TiledPanes {
                             self.session_is_mirrored,
                         );
                     }
-                    pane_contents_and_ui.render_terminal_title_if_needed(*client_id, client_mode);
+                    pane_contents_and_ui.render_terminal_title_if_needed(
+                        *client_id,
+                        client_mode,
+                        &mut self.window_title,
+                    );
                     // this is done for panes that don't have their own cursor (eg. panes of
                     // another user)
                     pane_contents_and_ui
@@ -458,14 +467,13 @@ impl TiledPanes {
             }
         }
         // render boundaries if needed
-        for (client_id, boundaries) in &mut client_id_to_boundaries {
-            // TODO: add some conditional rendering here so this isn't rendered for every character
+        for (client_id, boundaries) in client_id_to_boundaries {
+            let boundaries_to_render = boundaries
+                .render(self.client_id_to_boundaries.get(&client_id))
+                .with_context(err_context)?;
+            self.client_id_to_boundaries.insert(client_id, boundaries);
             output
-                .add_character_chunks_to_client(
-                    *client_id,
-                    boundaries.render().with_context(err_context)?,
-                    None,
-                )
+                .add_character_chunks_to_client(client_id, boundaries_to_render, None)
                 .with_context(err_context)?;
         }
         Ok(())
@@ -513,7 +521,15 @@ impl TiledPanes {
         }
         self.set_pane_frames(self.draw_pane_frames);
     }
-    pub fn resize_active_pane_left(&mut self, client_id: ClientId) {
+
+    pub fn resize_active_pane(
+        &mut self,
+        client_id: ClientId,
+        strategy: &ResizeStrategy,
+    ) -> Result<()> {
+        let err_context =
+            || format!("failed to {strategy} for active tiled pane for client {client_id}");
+
         if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
             let mut pane_grid = TiledPaneGrid::new(
                 &mut self.panes,
@@ -521,82 +537,20 @@ impl TiledPanes {
                 *self.display_area.borrow(),
                 *self.viewport.borrow(),
             );
-            pane_grid.resize_pane_left(&active_pane_id);
+
+            pane_grid
+                .change_pane_size(&active_pane_id, strategy, (RESIZE_PERCENT, RESIZE_PERCENT))
+                .with_context(err_context)?;
+
             for pane in self.panes.values_mut() {
-                resize_pty!(pane, self.os_api).unwrap();
-            }
-        }
-    }
-    pub fn resize_active_pane_right(&mut self, client_id: ClientId) {
-        if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
-            let mut pane_grid = TiledPaneGrid::new(
-                &mut self.panes,
-                &self.panes_to_hide,
-                *self.display_area.borrow(),
-                *self.viewport.borrow(),
-            );
-            pane_grid.resize_pane_right(&active_pane_id);
-            for pane in self.panes.values_mut() {
-                resize_pty!(pane, self.os_api).unwrap();
-            }
-        }
-    }
-    pub fn resize_active_pane_up(&mut self, client_id: ClientId) {
-        if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
-            let mut pane_grid = TiledPaneGrid::new(
-                &mut self.panes,
-                &self.panes_to_hide,
-                *self.display_area.borrow(),
-                *self.viewport.borrow(),
-            );
-            pane_grid.resize_pane_up(&active_pane_id);
-            for pane in self.panes.values_mut() {
-                resize_pty!(pane, self.os_api).unwrap();
-            }
-        }
-    }
-    pub fn resize_active_pane_down(&mut self, client_id: ClientId) {
-        if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
-            let mut pane_grid = TiledPaneGrid::new(
-                &mut self.panes,
-                &self.panes_to_hide,
-                *self.display_area.borrow(),
-                *self.viewport.borrow(),
-            );
-            pane_grid.resize_pane_down(&active_pane_id);
-            for pane in self.panes.values_mut() {
-                resize_pty!(pane, self.os_api).unwrap();
-            }
-        }
-    }
-    pub fn resize_active_pane_increase(&mut self, client_id: ClientId) {
-        if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
-            let mut pane_grid = TiledPaneGrid::new(
-                &mut self.panes,
-                &self.panes_to_hide,
-                *self.display_area.borrow(),
-                *self.viewport.borrow(),
-            );
-            pane_grid.resize_increase(&active_pane_id);
-            for pane in self.panes.values_mut() {
-                resize_pty!(pane, self.os_api).unwrap();
-            }
-        }
-    }
-    pub fn resize_active_pane_decrease(&mut self, client_id: ClientId) {
-        if let Some(active_pane_id) = self.get_active_pane_id(client_id) {
-            let mut pane_grid = TiledPaneGrid::new(
-                &mut self.panes,
-                &self.panes_to_hide,
-                *self.display_area.borrow(),
-                *self.viewport.borrow(),
-            );
-            pane_grid.resize_decrease(&active_pane_id);
-            for pane in self.panes.values_mut() {
-                resize_pty!(pane, self.os_api).unwrap();
+                resize_pty!(pane, self.os_api, self.senders).unwrap();
             }
+            self.reset_boundaries();
         }
+
+        Ok(())
     }
+
     pub fn focus_next_pane(&mut self, client_id: ClientId) {
         let connected_clients: Vec<ClientId> =
             { self.connected_clients.borrow().iter().copied().collect() };
@@ -613,6 +567,7 @@ impl TiledPanes {
                 .insert(client_id, next_active_pane_id, &mut self.panes);
         }
         self.set_pane_active_at(next_active_pane_id);
+        self.reset_boundaries();
     }
     pub fn focus_previous_pane(&mut self, client_id: ClientId) {
         let connected_clients: Vec<ClientId> =
@@ -630,6 +585,7 @@ impl TiledPanes {
                 .insert(client_id, next_active_pane_id, &mut self.panes);
         }
         self.set_pane_active_at(next_active_pane_id);
+        self.reset_boundaries();
     }
     fn set_pane_active_at(&mut self, pane_id: PaneId) {
         if let Some(pane) = self.get_pane_mut(pane_id) {
@@ -826,7 +782,7 @@ impl TiledPanes {
         if let Some(geom) = prev_geom_override {
             new_position.set_geom_override(geom);
         }
-        resize_pty!(new_position, self.os_api).unwrap();
+        resize_pty!(new_position, self.os_api, self.senders).unwrap();
         new_position.set_should_render(true);
 
         let current_position = self.panes.get_mut(&active_pane_id).unwrap();
@@ -834,7 +790,7 @@ impl TiledPanes {
         if let Some(geom) = next_geom_override {
             current_position.set_geom_override(geom);
         }
-        resize_pty!(current_position, self.os_api).unwrap();
+        resize_pty!(current_position, self.os_api, self.senders).unwrap();
         current_position.set_should_render(true);
         self.set_pane_frames(self.draw_pane_frames);
     }
@@ -860,7 +816,7 @@ impl TiledPanes {
                 if let Some(geom) = prev_geom_override {
                     new_position.set_geom_override(geom);
                 }
-                resize_pty!(new_position, self.os_api).unwrap();
+                resize_pty!(new_position, self.os_api, self.senders).unwrap();
                 new_position.set_should_render(true);
 
                 let current_position = self.panes.get_mut(active_pane_id).unwrap();
@@ -868,7 +824,7 @@ impl TiledPanes {
                 if let Some(geom) = next_geom_override {
                     current_position.set_geom_override(geom);
                 }
-                resize_pty!(current_position, self.os_api).unwrap();
+                resize_pty!(current_position, self.os_api, self.senders).unwrap();
                 current_position.set_should_render(true);
                 self.set_pane_frames(self.draw_pane_frames);
             }
@@ -896,7 +852,7 @@ impl TiledPanes {
                 if let Some(geom) = prev_geom_override {
                     new_position.set_geom_override(geom);
                 }
-                resize_pty!(new_position, self.os_api).unwrap();
+                resize_pty!(new_position, self.os_api, self.senders).unwrap();
                 new_position.set_should_render(true);
 
                 let current_position = self.panes.get_mut(active_pane_id).unwrap();
@@ -904,7 +860,7 @@ impl TiledPanes {
                 if let Some(geom) = next_geom_override {
                     current_position.set_geom_override(geom);
                 }
-                resize_pty!(current_position, self.os_api).unwrap();
+                resize_pty!(current_position, self.os_api, self.senders).unwrap();
                 current_position.set_should_render(true);
                 self.set_pane_frames(self.draw_pane_frames);
             }
@@ -932,7 +888,7 @@ impl TiledPanes {
                 if let Some(geom) = prev_geom_override {
                     new_position.set_geom_override(geom);
                 }
-                resize_pty!(new_position, self.os_api).unwrap();
+                resize_pty!(new_position, self.os_api, self.senders).unwrap();
                 new_position.set_should_render(true);
 
                 let current_position = self.panes.get_mut(active_pane_id).unwrap();
@@ -940,7 +896,7 @@ impl TiledPanes {
                 if let Some(geom) = next_geom_override {
                     current_position.set_geom_override(geom);
                 }
-                resize_pty!(current_position, self.os_api).unwrap();
+                resize_pty!(current_position, self.os_api, self.senders).unwrap();
                 current_position.set_should_render(true);
                 self.set_pane_frames(self.draw_pane_frames);
             }
@@ -968,7 +924,7 @@ impl TiledPanes {
                 if let Some(geom) = prev_geom_override {
                     new_position.set_geom_override(geom);
                 }
-                resize_pty!(new_position, self.os_api).unwrap();
+                resize_pty!(new_position, self.os_api, self.senders).unwrap();
                 new_position.set_should_render(true);
 
                 let current_position = self.panes.get_mut(active_pane_id).unwrap();
@@ -976,7 +932,7 @@ impl TiledPanes {
                 if let Some(geom) = next_geom_override {
                     current_position.set_geom_override(geom);
                 }
-                resize_pty!(current_position, self.os_api).unwrap();
+                resize_pty!(current_position, self.os_api, self.senders).unwrap();
                 current_position.set_should_render(true);
                 self.set_pane_frames(self.draw_pane_frames);
             }
@@ -988,12 +944,20 @@ impl TiledPanes {
             .iter()
             .map(|(cid, pid)| (*cid, *pid))
             .collect();
-        let next_active_pane_id = self
+
+        // find the most recently active pane
+        let mut next_active_pane_candidates: Vec<(&PaneId, &Box<dyn Pane>)> = self
             .panes
             .iter()
-            .filter(|(p_id, _)| !self.panes_to_hide.contains(p_id))
-            .find(|(p_id, p)| **p_id != pane_id && p.selectable())
-            .map(|(p_id, _p)| *p_id);
+            .filter(|(p_id, p)| !self.panes_to_hide.contains(p_id) && p.selectable())
+            .collect();
+        next_active_pane_candidates.sort_by(|(_pane_id_a, pane_a), (_pane_id_b, pane_b)| {
+            pane_a.active_at().cmp(&pane_b.active_at())
+        });
+        let next_active_pane_id = next_active_pane_candidates
+            .last()
+            .map(|(pane_id, _pane)| **pane_id);
+
         match next_active_pane_id {
             Some(next_active_pane) => {
                 for (client_id, active_pane_id) in active_panes {
@@ -1007,6 +971,7 @@ impl TiledPanes {
         }
     }
     pub fn extract_pane(&mut self, pane_id: PaneId) -> Option<Box<dyn Pane>> {
+        self.reset_boundaries();
         self.panes.remove(&pane_id)
     }
     pub fn remove_pane(&mut self, pane_id: PaneId) -> Option<Box<dyn Pane>> {
@@ -1182,6 +1147,9 @@ impl TiledPanes {
                 .insert(client_id, to_pane_id, &mut self.panes);
         }
     }
+    fn reset_boundaries(&mut self) {
+        self.client_id_to_boundaries.clear();
+    }
 }
 
 #[allow(clippy::borrowed_box)]
