@@ -5,11 +5,16 @@ use std::collections::{BTreeMap, HashMap, HashSet};
 use std::rc::Rc;
 use std::str;
 
+use zellij_utils::data::{Direction, Resize, ResizeStrategy};
 use zellij_utils::errors::prelude::*;
 use zellij_utils::input::command::RunCommand;
 use zellij_utils::input::options::Clipboard;
 use zellij_utils::pane_size::{Size, SizeInPixels};
-use zellij_utils::{input::command::TerminalAction, input::layout::PaneLayout, position::Position};
+use zellij_utils::{
+    input::command::TerminalAction,
+    input::layout::{PaneLayout, RunPluginLocation},
+    position::Position,
+};
 
 use crate::panes::alacritty_functions::xparse_color;
 use crate::panes::terminal_character::AnsiCode;
@@ -18,11 +23,11 @@ use crate::{
     output::Output,
     panes::sixel::SixelImageStore,
     panes::PaneId,
+    plugins::PluginInstruction,
     pty::{ClientOrTabIndex, PtyInstruction, VteBytes},
     tab::Tab,
     thread_bus::Bus,
     ui::overlay::{Overlay, OverlayWindow, Overlayable},
-    wasm_vm::PluginInstruction,
     ClientId, ServerInstruction,
 };
 use zellij_utils::{
@@ -118,6 +123,7 @@ type HoldForCommand = Option<RunCommand>;
 #[derive(Debug, Clone)]
 pub enum ScreenInstruction {
     PtyBytes(u32, VteBytes),
+    PluginBytes(Vec<(u32, ClientId, VteBytes)>), // u32 is plugin_id
     Render,
     NewPane(
         PaneId,
@@ -132,12 +138,7 @@ pub enum ScreenInstruction {
     HorizontalSplit(PaneId, Option<InitialTitle>, HoldForCommand, ClientId),
     VerticalSplit(PaneId, Option<InitialTitle>, HoldForCommand, ClientId),
     WriteCharacter(Vec<u8>, ClientId),
-    ResizeLeft(ClientId),
-    ResizeRight(ClientId),
-    ResizeDown(ClientId),
-    ResizeUp(ClientId),
-    ResizeIncrease(ClientId),
-    ResizeDecrease(ClientId),
+    Resize(ClientId, ResizeStrategy),
     SwitchFocus(ClientId),
     FocusNextPane(ClientId),
     FocusPreviousPane(ClientId),
@@ -173,7 +174,19 @@ pub enum ScreenInstruction {
     HoldPane(PaneId, Option<i32>, RunCommand, Option<ClientId>), // Option<i32> is the exit status
     UpdatePaneName(Vec<u8>, ClientId),
     UndoRenamePane(ClientId),
-    NewTab(PaneLayout, Vec<(u32, HoldForCommand)>, ClientId),
+    NewTab(
+        Option<TerminalAction>,
+        Option<PaneLayout>,
+        Option<String>,
+        ClientId,
+    ),
+    ApplyLayout(
+        PaneLayout,
+        Vec<(u32, HoldForCommand)>,
+        HashMap<RunPluginLocation, Vec<u32>>,
+        usize, // tab_index
+        ClientId,
+    ),
     SwitchTabNext(ClientId),
     SwitchTabPrev(ClientId),
     ToggleActiveSyncTab(ClientId),
@@ -217,6 +230,7 @@ impl From<&ScreenInstruction> for ScreenContext {
     fn from(screen_instruction: &ScreenInstruction) -> Self {
         match *screen_instruction {
             ScreenInstruction::PtyBytes(..) => ScreenContext::HandlePtyBytes,
+            ScreenInstruction::PluginBytes(..) => ScreenContext::PluginBytes,
             ScreenInstruction::Render => ScreenContext::Render,
             ScreenInstruction::NewPane(..) => ScreenContext::NewPane,
             ScreenInstruction::OpenInPlaceEditor(..) => ScreenContext::OpenInPlaceEditor,
@@ -227,12 +241,30 @@ impl From<&ScreenInstruction> for ScreenContext {
             ScreenInstruction::HorizontalSplit(..) => ScreenContext::HorizontalSplit,
             ScreenInstruction::VerticalSplit(..) => ScreenContext::VerticalSplit,
             ScreenInstruction::WriteCharacter(..) => ScreenContext::WriteCharacter,
-            ScreenInstruction::ResizeLeft(..) => ScreenContext::ResizeLeft,
-            ScreenInstruction::ResizeRight(..) => ScreenContext::ResizeRight,
-            ScreenInstruction::ResizeDown(..) => ScreenContext::ResizeDown,
-            ScreenInstruction::ResizeUp(..) => ScreenContext::ResizeUp,
-            ScreenInstruction::ResizeIncrease(..) => ScreenContext::ResizeIncrease,
-            ScreenInstruction::ResizeDecrease(..) => ScreenContext::ResizeDecrease,
+            ScreenInstruction::Resize(.., strategy) => match strategy {
+                ResizeStrategy {
+                    resize: Resize::Increase,
+                    direction,
+                    ..
+                } => match direction {
+                    Some(Direction::Left) => ScreenContext::ResizeIncreaseLeft,
+                    Some(Direction::Down) => ScreenContext::ResizeIncreaseDown,
+                    Some(Direction::Up) => ScreenContext::ResizeIncreaseUp,
+                    Some(Direction::Right) => ScreenContext::ResizeIncreaseRight,
+                    None => ScreenContext::ResizeIncreaseAll,
+                },
+                ResizeStrategy {
+                    resize: Resize::Decrease,
+                    direction,
+                    ..
+                } => match direction {
+                    Some(Direction::Left) => ScreenContext::ResizeDecreaseLeft,
+                    Some(Direction::Down) => ScreenContext::ResizeDecreaseDown,
+                    Some(Direction::Up) => ScreenContext::ResizeDecreaseUp,
+                    Some(Direction::Right) => ScreenContext::ResizeDecreaseRight,
+                    None => ScreenContext::ResizeDecreaseAll,
+                },
+            },
             ScreenInstruction::SwitchFocus(..) => ScreenContext::SwitchFocus,
             ScreenInstruction::FocusNextPane(..) => ScreenContext::FocusNextPane,
             ScreenInstruction::FocusPreviousPane(..) => ScreenContext::FocusPreviousPane,
@@ -273,6 +305,7 @@ impl From<&ScreenInstruction> for ScreenContext {
             ScreenInstruction::UpdatePaneName(..) => ScreenContext::UpdatePaneName,
             ScreenInstruction::UndoRenamePane(..) => ScreenContext::UndoRenamePane,
             ScreenInstruction::NewTab(..) => ScreenContext::NewTab,
+            ScreenInstruction::ApplyLayout(..) => ScreenContext::ApplyLayout,
             ScreenInstruction::SwitchTabNext(..) => ScreenContext::SwitchTabNext,
             ScreenInstruction::SwitchTabPrev(..) => ScreenContext::SwitchTabPrev,
             ScreenInstruction::CloseTab(..) => ScreenContext::CloseTab,
@@ -531,7 +564,6 @@ impl Screen {
         };
 
         if let Some(new_tab) = self.tabs.values().find(|t| t.position == new_tab_pos) {
-            //if let Some(current_tab) = self.get_active_tab(client_id) {
             match self.get_active_tab(client_id) {
                 Ok(current_tab) => {
                     // If new active tab is same as the current one, do nothing.
@@ -760,18 +792,22 @@ impl Screen {
                 let vte_overlay = overlay.generate_overlay(size).context(err_context)?;
                 tab.render(&mut output, Some(vte_overlay))
                     .context(err_context)?;
-            } else {
+            } else if !tab.is_pending() {
                 tabs_to_close.push(*tab_index);
             }
         }
         for tab_index in tabs_to_close {
             self.close_tab_at_index(tab_index).context(err_context)?;
         }
-        let serialized_output = output.serialize().context(err_context)?;
-        self.bus
-            .senders
-            .send_to_server(ServerInstruction::Render(Some(serialized_output)))
-            .context(err_context)
+        if output.is_dirty() {
+            let serialized_output = output.serialize().context(err_context)?;
+            self.bus
+                .senders
+                .send_to_server(ServerInstruction::Render(Some(serialized_output)))
+                .context(err_context)
+        } else {
+            Ok(())
+        }
     }
 
     /// Returns a mutable reference to this [`Screen`]'s tabs.
@@ -833,14 +869,8 @@ impl Screen {
         self.get_tabs_mut().get_mut(&tab_index)
     }
 
-    /// Creates a new [`Tab`] in this [`Screen`], applying the specified [`Layout`]
-    /// and switching to it.
-    pub fn new_tab(
-        &mut self,
-        layout: PaneLayout,
-        new_ids: Vec<(u32, HoldForCommand)>,
-        client_id: ClientId,
-    ) -> Result<()> {
+    /// Creates a new [`Tab`] in this [`Screen`]
+    pub fn new_tab(&mut self, tab_index: usize, client_id: ClientId) -> Result<()> {
         let err_context = || format!("failed to create new tab for client {client_id:?}",);
 
         let client_id = if self.get_active_tab(client_id).is_ok() {
@@ -851,9 +881,8 @@ impl Screen {
             client_id
         };
 
-        let tab_index = self.get_new_tab_index();
         let position = self.tabs.len();
-        let mut tab = Tab::new(
+        let tab = Tab::new(
             tab_index,
             position,
             String::new(),
@@ -877,35 +906,72 @@ impl Screen {
             self.terminal_emulator_colors.clone(),
             self.terminal_emulator_color_codes.clone(),
         );
-        tab.apply_layout(layout, new_ids, tab_index, client_id)
-            .with_context(err_context)?;
-        if self.session_is_mirrored {
-            if let Ok(active_tab) = self.get_active_tab_mut(client_id) {
-                let client_mode_infos_in_source_tab = active_tab.drain_connected_clients(None);
-                tab.add_multiple_clients(client_mode_infos_in_source_tab)
-                    .with_context(err_context)?;
-                if active_tab.has_no_connected_clients() {
-                    active_tab.visible(false).with_context(err_context)?;
-                }
-            }
+        self.tabs.insert(tab_index, tab);
+        Ok(())
+    }
+    pub fn apply_layout(
+        &mut self,
+        layout: PaneLayout,
+        new_terminal_ids: Vec<(u32, HoldForCommand)>,
+        new_plugin_ids: HashMap<RunPluginLocation, Vec<u32>>,
+        tab_index: usize,
+        client_id: ClientId,
+    ) -> Result<()> {
+        let client_id = if self.get_active_tab(client_id).is_ok() {
+            client_id
+        } else if let Some(first_client_id) = self.get_first_client_id() {
+            first_client_id
+        } else {
+            client_id
+        };
+        let err_context = || format!("failed to apply layout for tab {tab_index:?}",);
+
+        // move the relevant clients out of the current tab and place them in the new one
+        let drained_clients = if self.session_is_mirrored {
+            let client_mode_infos_in_source_tab =
+                if let Ok(active_tab) = self.get_active_tab_mut(client_id) {
+                    let client_mode_infos_in_source_tab = active_tab.drain_connected_clients(None);
+                    if active_tab.has_no_connected_clients() {
+                        active_tab.visible(false).with_context(err_context)?;
+                    }
+                    Some(client_mode_infos_in_source_tab)
+                } else {
+                    None
+                };
             let all_connected_clients: Vec<ClientId> =
                 self.connected_clients.borrow().iter().copied().collect();
             for client_id in all_connected_clients {
                 self.update_client_tab_focus(client_id, tab_index);
             }
+            client_mode_infos_in_source_tab
         } else if let Ok(active_tab) = self.get_active_tab_mut(client_id) {
             let client_mode_info_in_source_tab =
                 active_tab.drain_connected_clients(Some(vec![client_id]));
-            tab.add_multiple_clients(client_mode_info_in_source_tab)
-                .with_context(err_context)?;
             if active_tab.has_no_connected_clients() {
                 active_tab.visible(false).with_context(err_context)?;
             }
             self.update_client_tab_focus(client_id, tab_index);
-        }
+            Some(client_mode_info_in_source_tab)
+        } else {
+            None
+        };
+
+        // apply the layout to the new tab
+        let tab = self.tabs.get_mut(&tab_index).unwrap(); // TODO: no unwrap
+        tab.apply_layout(
+            layout,
+            new_terminal_ids,
+            new_plugin_ids,
+            tab_index,
+            client_id,
+        )
+        .with_context(err_context)?;
         tab.update_input_modes().with_context(err_context)?;
         tab.visible(true).with_context(err_context)?;
-        self.tabs.insert(tab_index, tab);
+        if let Some(drained_clients) = drained_clients {
+            tab.add_multiple_clients(drained_clients)
+                .with_context(err_context)?;
+        }
         if !self.active_tab_indices.contains_key(&client_id) {
             // this means this is a new client and we need to add it to our state properly
             self.add_client(client_id).with_context(err_context)?;
@@ -967,6 +1033,7 @@ impl Screen {
     }
 
     pub fn update_tabs(&self) -> Result<()> {
+        let mut plugin_updates = vec![];
         for (client_id, active_tab_index) in self.active_tab_indices.iter() {
             let mut tab_data = vec![];
             for tab in self.tabs.values() {
@@ -993,15 +1060,12 @@ impl Screen {
                     other_focused_clients,
                 });
             }
-            self.bus
-                .senders
-                .send_to_plugin(PluginInstruction::Update(
-                    None,
-                    Some(*client_id),
-                    Event::TabUpdate(tab_data),
-                ))
-                .context("failed to update tabs")?;
+            plugin_updates.push((None, Some(*client_id), Event::TabUpdate(tab_data)));
         }
+        self.bus
+            .senders
+            .send_to_plugin(PluginInstruction::Update(plugin_updates))
+            .context("failed to update tabs")?;
         Ok(())
     }
 
@@ -1276,6 +1340,19 @@ pub(crate) fn screen_thread_main(
                     }
                 }
             },
+            ScreenInstruction::PluginBytes(mut plugin_bytes) => {
+                for (pid, client_id, vte_bytes) in plugin_bytes.drain(..) {
+                    let all_tabs = screen.get_tabs_mut();
+                    for tab in all_tabs.values_mut() {
+                        if tab.has_plugin(pid) {
+                            tab.handle_plugin_bytes(pid, client_id, vte_bytes)
+                                .context("failed to process plugin bytes")?;
+                            break;
+                        }
+                    }
+                }
+                screen.render()?;
+            },
             ScreenInstruction::Render => {
                 screen.render()?;
             },
@@ -1428,56 +1505,12 @@ pub(crate) fn screen_thread_main(
                     screen.update_tabs()?;
                 }
             },
-            ScreenInstruction::ResizeLeft(client_id) => {
+            ScreenInstruction::Resize(client_id, strategy) => {
                 active_tab_and_connected_client_id!(
                     screen,
                     client_id,
-                    |tab: &mut Tab, client_id: ClientId| tab.resize_left(client_id)
-                );
-                screen.unblock_input()?;
-                screen.render()?;
-            },
-            ScreenInstruction::ResizeRight(client_id) => {
-                active_tab_and_connected_client_id!(
-                    screen,
-                    client_id,
-                    |tab: &mut Tab, client_id: ClientId| tab.resize_right(client_id)
-                );
-                screen.unblock_input()?;
-                screen.render()?;
-            },
-            ScreenInstruction::ResizeDown(client_id) => {
-                active_tab_and_connected_client_id!(
-                    screen,
-                    client_id,
-                    |tab: &mut Tab, client_id: ClientId| tab.resize_down(client_id)
-                );
-                screen.unblock_input()?;
-                screen.render()?;
-            },
-            ScreenInstruction::ResizeUp(client_id) => {
-                active_tab_and_connected_client_id!(
-                    screen,
-                    client_id,
-                    |tab: &mut Tab, client_id: ClientId| tab.resize_up(client_id)
-                );
-                screen.unblock_input()?;
-                screen.render()?;
-            },
-            ScreenInstruction::ResizeIncrease(client_id) => {
-                active_tab_and_connected_client_id!(
-                    screen,
-                    client_id,
-                    |tab: &mut Tab, client_id: ClientId| tab.resize_increase(client_id)
-                );
-                screen.unblock_input()?;
-                screen.render()?;
-            },
-            ScreenInstruction::ResizeDecrease(client_id) => {
-                active_tab_and_connected_client_id!(
-                    screen,
-                    client_id,
-                    |tab: &mut Tab, client_id: ClientId| tab.resize_decrease(client_id)
+                    |tab: &mut Tab, client_id: ClientId| tab.resize(client_id, strategy),
+                    ?
                 );
                 screen.unblock_input()?;
                 screen.render()?;
@@ -1837,8 +1870,28 @@ pub(crate) fn screen_thread_main(
                 screen.unblock_input()?;
                 screen.render()?;
             },
-            ScreenInstruction::NewTab(layout, new_pane_pids, client_id) => {
-                screen.new_tab(layout, new_pane_pids, client_id)?;
+            ScreenInstruction::NewTab(default_shell, layout, tab_name, client_id) => {
+                let tab_index = screen.get_new_tab_index();
+                screen.new_tab(tab_index, client_id)?;
+                screen
+                    .bus
+                    .senders
+                    .send_to_plugin(PluginInstruction::NewTab(
+                        default_shell,
+                        layout,
+                        tab_name,
+                        tab_index,
+                        client_id,
+                    ))?;
+            },
+            ScreenInstruction::ApplyLayout(
+                layout,
+                new_pane_pids,
+                new_plugin_ids,
+                tab_index,
+                client_id,
+            ) => {
+                screen.apply_layout(layout, new_pane_pids, new_plugin_ids, tab_index, client_id)?;
                 screen.unblock_input()?;
                 screen.render()?;
             },
