@@ -1,8 +1,7 @@
-use crate::data::InputMode;
+use crate::data::{Direction, InputMode, Resize};
 use crate::setup::Setup;
 use crate::{
     consts::{ZELLIJ_CONFIG_DIR_ENV, ZELLIJ_CONFIG_FILE_ENV},
-    input::actions::{Direction, ResizeDirection},
     input::options::CliOptions,
 };
 use clap::{Parser, Subcommand};
@@ -184,8 +183,11 @@ pub enum CliAction {
     Write { bytes: Vec<u8> },
     /// Write characters to the terminal.
     WriteChars { chars: String },
-    /// Resize the focused pane in the specified direction. [right|left|up|down|+|-]
-    Resize { resize_direction: ResizeDirection },
+    /// [increase|decrease] the focused panes area at the [left|down|up|right] border.
+    Resize {
+        resize: Resize,
+        direction: Option<Direction>,
+    },
     /// Change focus to the next pane
     FocusNextPane,
     /// Change focus to the previous pane
