@@ -14,6 +14,7 @@ use colored::*;
 use log::error;
 use serde::{Deserialize, Serialize};
 use std::fmt::{Display, Error, Formatter};
+use std::path::PathBuf;
 
 use miette::Diagnostic;
 use thiserror::Error as ThisError;
@@ -92,11 +93,36 @@ pub trait LoggableError<T>: Sized {
     ///     .print_error(|msg| println!("{msg}"))
     ///     .unwrap();
     /// ```
+    #[track_caller]
     fn print_error<F: Fn(&str)>(self, fun: F) -> Self;
 
-    /// Convenienve function, calls `print_error` with the closure `|msg| log::error!("{}", msg)`.
+    /// Convenienve function, calls `print_error` and logs the result as error.
+    ///
+    /// This is not a wrapper around `log::error!`, because the `log` crate uses a lot of compile
+    /// time macros from `std` to determine caller locations/module names etc. Since these are
+    /// resolved at compile time in the location they are written, they would always resolve to the
+    /// location in this function where `log::error!` is called, masking the real caller location.
+    /// Hence, we build the log message ourselves. This means that we lose the information about
+    /// the calling module (Because it can only be resolved at compile time), however the callers
+    /// file and line number are preserved.
+    #[track_caller]
     fn to_log(self) -> Self {
-        self.print_error(|msg| log::error!("{}", msg))
+        let caller = std::panic::Location::caller();
+        self.print_error(|msg| {
+            // Build the log entry manually
+            // NOTE: The log entry has no module path associated with it. This is because `log`
+            // gets the module path from the `std::module_path!()` macro, which is replaced at
+            // compile time in the location it is written!
+            log::logger().log(
+                &log::Record::builder()
+                    .level(log::Level::Error)
+                    .args(format_args!("{}", msg))
+                    .file(Some(caller.file()))
+                    .line(Some(caller.line()))
+                    .module_path(None)
+                    .build(),
+            );
+        })
     }
 
     /// Convenienve function, calls `print_error` with the closure `|msg| eprintln!("{}", msg)`.
@@ -133,6 +159,7 @@ pub trait FatalError<T> {
     /// Discards the result type afterwards.
     ///
     /// [`to_log`]: LoggableError::to_log
+    #[track_caller]
     fn non_fatal(self);
 
     /// Mark results as being fatal.
@@ -218,6 +245,7 @@ impl Display for ContextType {
 #[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
 pub enum ScreenContext {
     HandlePtyBytes,
+    PluginBytes,
     Render,
     NewPane,
     OpenInPlaceEditor,
@@ -228,6 +256,16 @@ pub enum ScreenContext {
     HorizontalSplit,
     VerticalSplit,
     WriteCharacter,
+    ResizeIncreaseAll,
+    ResizeIncreaseLeft,
+    ResizeIncreaseDown,
+    ResizeIncreaseUp,
+    ResizeIncreaseRight,
+    ResizeDecreaseAll,
+    ResizeDecreaseLeft,
+    ResizeDecreaseDown,
+    ResizeDecreaseUp,
+    ResizeDecreaseRight,
     ResizeLeft,
     ResizeRight,
     ResizeDown,
@@ -275,6 +313,7 @@ pub enum ScreenContext {
     UpdatePaneName,
     UndoRenamePane,
     NewTab,
+    ApplyLayout,
     SwitchTabNext,
     SwitchTabPrev,
     CloseTab,
@@ -336,9 +375,11 @@ pub enum PluginContext {
     Update,
     Render,
     Unload,
+    Resize,
     Exit,
     AddClient,
     RemoveClient,
+    NewTab,
 }
 
 /// Stack call representations corresponding to the different types of [`ClientInstruction`]s.
@@ -353,6 +394,8 @@ pub enum ClientContext {
     Connected,
     ActiveClients,
     OwnClientId,
+    StartedParsingStdinQuery,
+    DoneParsingStdinQuery,
 }
 
 /// Stack call representations corresponding to the different types of [`ServerInstruction`]s.
@@ -392,6 +435,56 @@ pub enum ZellijError {
     #[error("failed to start PTY")]
     FailedToStartPty,
 
+    #[error(
+        "This version of zellij was built to load the core plugins from
+the globally configured plugin directory. However, a plugin wasn't found:
+
+    Plugin name: '{plugin_path}'
+    Plugin directory: '{plugin_dir}'
+
+If you're a user:
+    Please report this error to the distributor of your current zellij version
+
+If you're a developer:
+    Either make sure to include the plugins with the application (See feature
+    'disable_automatic_asset_installation'), or make them available in the
+    plugin directory.
+
+Possible fix for your problem:
+    Run `zellij setup --dump-plugins`, and optionally point it to your
+    'DATA DIR', visible in e.g. the output of `zellij setup --check`. Without
+    further arguments, it will use the default 'DATA DIR'.
+"
+    )]
+    BuiltinPluginMissing {
+        plugin_path: PathBuf,
+        plugin_dir: PathBuf,
+        #[source]
+        source: anyhow::Error,
+    },
+
+    #[error(
+        "It seems you tried to load the following builtin plugin:
+
+    Plugin name: '{plugin_path}'
+
+This is not a builtin plugin known to this version of zellij. If you were using
+a custom layout, please refer to the layout documentation at:
+
+    https://zellij.dev/documentation/creating-a-layout.html#plugin
+
+If you think this is a bug and the plugin is indeed an internal plugin, please
+open an issue on GitHub:
+
+    https://github.com/zellij-org/zellij/issues
+"
+    )]
+    BuiltinPluginNonexistent {
+        plugin_path: PathBuf,
+        #[source]
+        source: anyhow::Error,
+    },
+
     #[error("an error occured")]
     GenericError { source: anyhow::Error },
 }
@@ -565,13 +658,14 @@ mod not_wasm {
                 Err(e) => {
                     let (msg, context) = e.into_inner();
                     if *crate::consts::DEBUG_MODE.get().unwrap_or(&true) {
-                        Err(
-                            crate::anyhow::anyhow!("failed to send message to channel: {:#?}", msg)
-                                .context(context.to_string()),
-                        )
+                        Err(crate::anyhow::anyhow!(
+                            "failed to send message to channel: {:#?}",
+                            msg
+                        ))
+                        .with_context(|| context.to_string())
                     } else {
-                        Err(crate::anyhow::anyhow!("failed to send message to channel")
-                            .context(context.to_string()))
+                        Err(crate::anyhow::anyhow!("failed to send message to channel"))
+                            .with_context(|| context.to_string())
                     }
                 },
             }
