@@ -4,6 +4,7 @@ use super::command::RunCommandAction;
 use super::layout::{Layout, PaneLayout};
 use crate::cli::CliAction;
 use crate::data::InputMode;
+use crate::data::{Direction, Resize};
 use crate::input::config::{ConfigError, KdlError};
 use crate::input::options::OnForceClose;
 use miette::{NamedSource, Report};
@@ -14,30 +15,6 @@ use std::str::FromStr;
 
 use crate::position::Position;
 
-/// The four directions (left, right, up, down).
-#[derive(Eq, Clone, Copy, Debug, PartialEq, Deserialize, Serialize)]
-pub enum Direction {
-    Left,
-    Right,
-    Up,
-    Down,
-}
-impl FromStr for Direction {
-    type Err = String;
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s {
-            "Left" | "left" => Ok(Direction::Left),
-            "Right" | "right" => Ok(Direction::Right),
-            "Up" | "up" => Ok(Direction::Up),
-            "Down" | "down" => Ok(Direction::Down),
-            _ => Err(format!(
-                "Failed to parse Direction. Unknown Direction: {}",
-                s
-            )),
-        }
-    }
-}
-
 #[derive(Clone, Debug, PartialEq, Eq, Deserialize, Serialize)]
 pub enum ResizeDirection {
     Left,
@@ -126,8 +103,8 @@ pub enum Action {
     SwitchToMode(InputMode),
     /// Switch all connected clients to the specified input mode.
     SwitchModeForAllClients(InputMode),
-    /// Resize focus pane in specified direction.
-    Resize(ResizeDirection),
+    /// Shrink/enlarge focused pane at specified border
+    Resize(Resize, Option<Direction>),
     /// Switch focus to next pane in specified direction.
     FocusNextPane,
     FocusPreviousPane,
@@ -228,11 +205,14 @@ pub enum Action {
 }
 
 impl Action {
-    pub fn actions_from_cli(cli_action: CliAction) -> Result<Vec<Action>, String> {
+    pub fn actions_from_cli(
+        cli_action: CliAction,
+        get_current_dir: Box<dyn Fn() -> PathBuf>,
+    ) -> Result<Vec<Action>, String> {
         match cli_action {
             CliAction::Write { bytes } => Ok(vec![Action::Write(bytes)]),
             CliAction::WriteChars { chars } => Ok(vec![Action::WriteChars(chars)]),
-            CliAction::Resize { resize_direction } => Ok(vec![Action::Resize(resize_direction)]),
+            CliAction::Resize { resize, direction } => Ok(vec![Action::Resize(resize, direction)]),
             CliAction::FocusNextPane => Ok(vec![Action::FocusNextPane]),
             CliAction::FocusPreviousPane => Ok(vec![Action::FocusPreviousPane]),
             CliAction::MoveFocus { direction } => Ok(vec![Action::MoveFocus(direction)]),
@@ -265,7 +245,10 @@ impl Action {
                 if !command.is_empty() {
                     let mut command = command.clone();
                     let (command, args) = (PathBuf::from(command.remove(0)), command);
-                    let cwd = cwd.or_else(|| std::env::current_dir().ok());
+                    let current_dir = get_current_dir();
+                    let cwd = cwd
+                        .map(|cwd| current_dir.join(cwd))
+                        .or_else(|| Some(current_dir));
                     let hold_on_start = start_suspended;
                     let hold_on_close = !close_on_exit;
                     let run_command_action = RunCommandAction {
@@ -304,7 +287,10 @@ impl Action {
                 cwd,
             } => {
                 let mut file = file;
-                let cwd = cwd.or_else(|| std::env::current_dir().ok());
+                let current_dir = get_current_dir();
+                let cwd = cwd
+                    .map(|cwd| current_dir.join(cwd))
+                    .or_else(|| Some(current_dir));
                 if file.is_relative() {
                     if let Some(cwd) = cwd {
                         file = cwd.join(file);
@@ -338,6 +324,10 @@ impl Action {
             ]),
             CliAction::UndoRenameTab => Ok(vec![Action::UndoRenameTab]),
             CliAction::NewTab { name, layout, cwd } => {
+                let current_dir = get_current_dir();
+                let cwd = cwd
+                    .map(|cwd| current_dir.join(cwd))
+                    .or_else(|| Some(current_dir));
                 if let Some(layout_path) = layout {
                     let (path_to_raw_layout, raw_layout) =
                         Layout::stringified_from_path_or_default(Some(&layout_path), None)
