@@ -9,6 +9,7 @@
 //  If plugins should be able to depend on the layout system
 //  then [`zellij-utils`] could be a proper place.
 use crate::{
+    data::Direction,
     input::{
         command::RunCommand,
         config::{Config, ConfigError},
@@ -48,6 +49,15 @@ impl Not for SplitDirection {
     }
 }
 
+impl From<Direction> for SplitDirection {
+    fn from(direction: Direction) -> Self {
+        match direction {
+            Direction::Left | Direction::Right => SplitDirection::Horizontal,
+            Direction::Down | Direction::Up => SplitDirection::Vertical,
+        }
+    }
+}
+
 #[derive(Debug, Serialize, Deserialize, Clone, Copy, PartialEq, Eq)]
 pub enum SplitSize {
     #[serde(alias = "percent")]
@@ -168,7 +178,7 @@ pub struct RunPlugin {
     pub location: RunPluginLocation,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq)]
+#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq, Hash)]
 pub enum RunPluginLocation {
     File(PathBuf),
     Zellij(PluginTag),
