@@ -268,6 +268,32 @@ fn layout_with_command_panes_and_cwd_and_args() {
     assert_eq!(layout, expected_layout);
 }
 
+#[test]
+fn layout_with_command_panes_and_close_on_exit() {
+    let kdl_layout = r#"
+        layout {
+            pane command="htop" {
+                close_on_exit true
+            }
+        }
+    "#;
+    let layout = Layout::from_kdl(kdl_layout, "layout_file_name".into(), None).unwrap();
+    assert_snapshot!(format!("{:#?}", layout));
+}
+
+#[test]
+fn layout_with_command_panes_and_start_suspended() {
+    let kdl_layout = r#"
+        layout {
+            pane command="htop" {
+                start_suspended true
+            }
+        }
+    "#;
+    let layout = Layout::from_kdl(kdl_layout, "layout_file_name".into(), None).unwrap();
+    assert_snapshot!(format!("{:#?}", layout));
+}
+
 #[test]
 fn layout_with_plugin_panes() {
     let kdl_layout = r#"
@@ -1033,6 +1059,24 @@ fn args_override_args_in_template() {
     assert_snapshot!(format!("{:#?}", layout));
 }
 
+#[test]
+fn close_on_exit_overrides_close_on_exit_in_template() {
+    let kdl_layout = r#"
+        layout {
+            pane_template name="tail" {
+                command "tail"
+                close_on_exit false
+            }
+            tail
+            tail {
+                close_on_exit true
+            }
+        }
+    "#;
+    let layout = Layout::from_kdl(kdl_layout, "layout_file_name".into(), None).unwrap();
+    assert_snapshot!(format!("{:#?}", layout));
+}
+
 #[test]
 fn args_added_to_args_in_template() {
     let kdl_layout = r#"
@@ -1050,6 +1094,23 @@ fn args_added_to_args_in_template() {
     assert_snapshot!(format!("{:#?}", layout));
 }
 
+#[test]
+fn close_on_exit_added_to_close_on_exit_in_template() {
+    let kdl_layout = r#"
+        layout {
+            pane_template name="tail" {
+                command "tail"
+            }
+            tail
+            tail {
+                close_on_exit true
+            }
+        }
+    "#;
+    let layout = Layout::from_kdl(kdl_layout, "layout_file_name".into(), None).unwrap();
+    assert_snapshot!(format!("{:#?}", layout));
+}
+
 #[test]
 fn cwd_override_cwd_in_template() {
     let kdl_layout = r#"
@@ -1125,6 +1186,19 @@ fn error_on_bare_args_without_command() {
     assert!(layout.is_err(), "error provided");
 }
 
+#[test]
+fn error_on_bare_close_on_exit_without_command() {
+    let kdl_layout = r#"
+        layout {
+            pane {
+                close_on_exit true
+            }
+        }
+    "#;
+    let layout = Layout::from_kdl(kdl_layout, "layout_file_name".into(), None);
+    assert!(layout.is_err(), "error provided");
+}
+
 #[test]
 fn error_on_bare_args_in_template_without_command() {
     let kdl_layout = r#"
@@ -1139,6 +1213,20 @@ fn error_on_bare_args_in_template_without_command() {
     assert!(layout.is_err(), "error provided");
 }
 
+#[test]
+fn error_on_bare_close_on_exit_in_template_without_command() {
+    let kdl_layout = r#"
+        layout {
+            pane_template name="my_template"
+            my_template {
+                close_on_exit true
+            }
+        }
+    "#;
+    let layout = Layout::from_kdl(kdl_layout, "layout_file_name".into(), None);
+    assert!(layout.is_err(), "error provided");
+}
+
 #[test]
 fn pane_template_command_with_cwd_overriden_by_its_consumers_command_cwd() {
     let kdl_layout = r#"
